import React, { useState, useEffect } from 'react';
import { Plus, Filter, Calendar, Scissors, AlertTriangle, Square, CheckCircle, Clock, Map } from 'lucide-react';
import { Zone } from '../types/zone';
import { api } from '../services/api';
import ZoneCard from './ZoneCard';
import ZoneForm from './ZoneForm';
import SitePlan from './SitePlan';

type FilterType = 'all' | 'due' | 'overdue';
type ViewType = 'dashboard' | 'sitePlan';

const Dashboard: React.FC = () => {
  const [zones, setZones] = useState<Zone[]>([]);
  const [filteredZones, setFilteredZones] = useState<Zone[]>([]);
  const [filter, setFilter] = useState<FilterType>('all');
  const [view, setView] = useState<ViewType>('dashboard');
  const [showForm, setShowForm] = useState(false);
  const [editingZone, setEditingZone] = useState<Zone | null>(null);
  const [selectedZoneId, setSelectedZoneId] = useState<number | undefined>();
  const [loading, setLoading] = useState(true);

  useEffect(() => {
    loadZones();
  }, []);

  useEffect(() => {
    applyFilter();
  }, [zones, filter]);

  const loadZones = async () => {
    try {
      const data = await api.getZones();
      setZones(data);
    } catch (error) {
      console.error('Failed to load zones:', error);
    } finally {
      setLoading(false);
    }
  };

  const applyFilter = () => {
    let filtered = zones;
    switch (filter) {
      case 'due':
        filtered = zones.filter(zone => zone.isDueToday);
        break;
      case 'overdue':
        filtered = zones.filter(zone => zone.isOverdue);
        break;
      default:
        filtered = zones;
    }
    setFilteredZones(filtered);
  };

  const handleMarkAsMowed = async (id: number) => {
    try {
      await api.markAsMowed(id);
      loadZones();
    } catch (error) {
      console.error('Failed to mark as mowed:', error);
    }
  };

  const handleDeleteZone = async (id: number) => {
    if (window.confirm('Are you sure you want to delete this zone?')) {
      try {
        await api.deleteZone(id);
        loadZones();
        if (selectedZoneId === id) {
          setSelectedZoneId(undefined);
        }
      } catch (error) {
        console.error('Failed to delete zone:', error);
      }
    }
  };

  const handleFormSubmit = async () => {
    setShowForm(false);
    setEditingZone(null);
    loadZones();
  };

  const handleZoneSelect = (zone: Zone) => {
    setSelectedZoneId(zone.id);
    setView('dashboard');
    // Scroll to the zone card
    setTimeout(() => {
      const element = document.getElementById(`zone-card-${zone.id}`);
      if (element) {
        element.scrollIntoView({ behavior: 'smooth', block: 'center' });
      }
    }, 100);
  };

  // Calculate area statistics
  const overdueCount = zones.filter(zone => zone.isOverdue).length;
  const dueCount = zones.filter(zone => zone.isDueToday).length;
  const okCount = zones.filter(zone => zone.status === 'ok').length;
  const totalArea = zones.reduce((sum, zone) => sum + zone.area, 0);
  
  // Calculate mowed vs remaining area based on status
  const mowedArea = zones
    .filter(zone => zone.status === 'ok')
    .reduce((sum, zone) => sum + zone.area, 0);
  
  const remainingArea = zones
    .filter(zone => zone.status === 'due' || zone.status === 'overdue')
    .reduce((sum, zone) => sum + zone.area, 0);

  const mowedPercentage = totalArea > 0 ? (mowedArea / totalArea) * 100 : 0;

  if (loading) {
    return (
      <div className="min-h-screen bg-gray-50 flex items-center justify-center">
        <div className="animate-spin rounded-full h-12 w-12 border-b-2 border-green-600"></div>
      </div>
    );
  }

  return (
    <div className="min-h-screen bg-gray-50">
      <div className="max-w-7xl mx-auto px-4 sm:px-6 lg:px-8 py-8">
        {/* Header */}
        <div className="mb-8">
          <div className="flex items-center justify-between">
            <div>
              <h1 className="text-3xl font-bold text-gray-900 flex items-center gap-3">
                <Scissors className="h-8 w-8 text-green-600" />
                Lawn Care Manager
              </h1>
              <p className="mt-2 text-gray-600">Keep track of your lawn mowing schedule</p>
            </div>
            <div className="flex items-center gap-3">
              {/* View Toggle */}
              <div className="flex bg-gray-100 rounded-lg p-1">
                <button
                  onClick={() => setView('dashboard')}
                  className={`px-4 py-2 rounded-md text-sm font-medium transition-colors duration-200 ${
                    view === 'dashboard'
                      ? 'bg-white text-gray-900 shadow-sm'
                      : 'text-gray-600 hover:text-gray-900'
                  }`}
                >
                  <Calendar className="h-4 w-4 inline mr-2" />
                  Dashboard
                </button>
                <button
                  onClick={() => setView('sitePlan')}
                  className={`px-4 py-2 rounded-md text-sm font-medium transition-colors duration-200 ${
                    view === 'sitePlan'
                      ? 'bg-white text-gray-900 shadow-sm'
                      : 'text-gray-600 hover:text-gray-900'
                  }`}
                >
                  <Map className="h-4 w-4 inline mr-2" />
                  Site Plan
                </button>
              </div>
              
              <button
                onClick={() => setShowForm(true)}
                className="bg-green-600 hover:bg-green-700 text-white px-6 py-3 rounded-lg flex items-center gap-2 transition-colors duration-200 shadow-lg hover:shadow-xl"
              >
                <Plus className="h-5 w-5" />
                Add Zone
              </button>
            </div>
          </div>
        </div>

        {view === 'sitePlan' ? (
          <SitePlan 
            zones={zones} 
            onZoneSelect={handleZoneSelect}
            selectedZoneId={selectedZoneId}
          />
        ) : (
          <>
            {/* Stats Cards */}
            <div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-3 xl:grid-cols-6 gap-6 mb-8">
              <div className="bg-white rounded-lg shadow-md p-6 border-l-4 border-green-500">
                <div className="flex items-center justify-between">
                  <div>
                    <p className="text-sm font-medium text-gray-600">Total Zones</p>
                    <p className="text-2xl font-bold text-gray-900">{zones.length}</p>
                  </div>
                  <Calendar className="h-8 w-8 text-green-500" />
                </div>
              </div>

              <div className="bg-white rounded-lg shadow-md p-6 border-l-4 border-purple-500">
                <div className="flex items-center justify-between">
                  <div>
                    <p className="text-sm font-medium text-gray-600">Total Area</p>
                    <p className="text-2xl font-bold text-gray-900">{totalArea.toLocaleString()}</p>
                    <p className="text-xs text-gray-500">sq ft</p>
                  </div>
                  <Square className="h-8 w-8 text-purple-500" />
                </div>
              </div>

              <div className="bg-white rounded-lg shadow-md p-6 border-l-4 border-emerald-500">
                <div className="flex items-center justify-between">
                  <div>
                    <p className="text-sm font-medium text-gray-600">Mowed Area</p>
                    <p className="text-2xl font-bold text-gray-900">{mowedArea.toLocaleString()}</p>
                    <p className="text-xs text-gray-500">sq ft ({mowedPercentage.toFixed(1)}%)</p>
                  </div>
                  <CheckCircle className="h-8 w-8 text-emerald-500" />
                </div>
              </div>

              <div className="bg-white rounded-lg shadow-md p-6 border-l-4 border-amber-500">
                <div className="flex items-center justify-between">
                  <div>
                    <p className="text-sm font-medium text-gray-600">Needs Mowing</p>
                    <p className="text-2xl font-bold text-gray-900">{remainingArea.toLocaleString()}</p>
                    <p className="text-xs text-gray-500">sq ft ({(100 - mowedPercentage).toFixed(1)}%)</p>
                  </div>
                  <Clock className="h-8 w-8 text-amber-500" />
                </div>
              </div>

              <div className="bg-white rounded-lg shadow-md p-6 border-l-4 border-orange-500">
                <div className="flex items-center justify-between">
                  <div>
                    <p className="text-sm font-medium text-gray-600">Due Today</p>
                    <p className="text-2xl font-bold text-gray-900">{dueCount}</p>
                  </div>
                  <Scissors className="h-8 w-8 text-orange-500" />
                </div>
              </div>

              <div className="bg-white rounded-lg shadow-md p-6 border-l-4 border-red-500">
                <div className="flex items-center justify-between">
                  <div>
                    <p className="text-sm font-medium text-gray-600">Overdue</p>
                    <p className="text-2xl font-bold text-gray-900">{overdueCount}</p>
                  </div>
                  <AlertTriangle className="h-8 w-8 text-red-500" />
                </div>
              </div>
            </div>

            {/* Progress Bar */}
            {totalArea > 0 && (
              <div className="bg-white rounded-lg shadow-md p-6 mb-8">
                <div className="flex items-center justify-between mb-4">
                  <h3 className="text-lg font-semibold text-gray-900">Mowing Progress</h3>
                  <span className="text-sm text-gray-600">
                    {mowedArea.toLocaleString()} / {totalArea.toLocaleString()} sq ft
                  </span>
                </div>
                <div className="w-full bg-gray-200 rounded-full h-4 mb-2">
                  <div 
                    className="bg-gradient-to-r from-green-500 to-emerald-600 h-4 rounded-full transition-all duration-500 ease-out"
                    style={{ width: `${mowedPercentage}%` }}
                  ></div>
                </div>
                <div className="flex justify-between text-sm text-gray-600">
                  <span className="flex items-center gap-1">
                    <CheckCircle className="h-4 w-4 text-emerald-500" />
                    {mowedPercentage.toFixed(1)}% Complete
                  </span>
                  <span className="flex items-center gap-1">
                    <Clock className="h-4 w-4 text-amber-500" />
                    {(100 - mowedPercentage).toFixed(1)}% Remaining
                  </span>
                </div>
              </div>
            )}

            {/* Filter Buttons */}
            <div className="mb-6">
              <div className="flex items-center gap-2">
                <Filter className="h-5 w-5 text-gray-500" />
                <span className="text-sm font-medium text-gray-700">Filter:</span>
                <div className="flex gap-2">
                  {[
                    { key: 'all' as FilterType, label: 'All Zones', count: zones.length },
                    { key: 'due' as FilterType, label: 'Due Today', count: dueCount },
                    { key: 'overdue' as FilterType, label: 'Overdue', count: overdueCount },
                  ].map(({ key, label, count }) => (
                    <button
                      key={key}
                      onClick={() => setFilter(key)}
                      className={`px-4 py-2 rounded-full text-sm font-medium transition-colors duration-200 ${
                        filter === key
                          ? 'bg-green-600 text-white shadow-md'
                          : 'bg-white text-gray-700 hover:bg-gray-50 border border-gray-300'
                      }`}
                    >
                      {label} ({count})
                    </button>
                  ))}
                </div>
              </div>
            </div>

            {/* Zones Grid */}
            {filteredZones.length === 0 ? (
              <div className="text-center py-12">
                <Scissors className="mx-auto h-12 w-12 text-gray-400" />
                <h3 className="mt-2 text-sm font-medium text-gray-900">No zones found</h3>
                <p className="mt-1 text-sm text-gray-500">
                  {filter === 'all' ? 'Get started by adding your first lawn zone.' : `No zones match the "${filter}" filter.`}
                </p>
              </div>
            ) : (
              <div className="grid grid-cols-1 md:grid-cols-2 lg:grid-cols-3 xl:grid-cols-4 gap-6">
                {filteredZones.map(zone => (
                  <div
                    key={zone.id}
                    id={`zone-card-${zone.id}`}
                    className={`transition-all duration-300 ${
                      selectedZoneId === zone.id 
                        ? 'ring-4 ring-blue-300 ring-opacity-50 scale-105' 
                        : ''
                    }`}
                  >
                    <ZoneCard
                      zone={zone}
                      onMarkAsMowed={handleMarkAsMowed}
                      onEdit={(zone) => {
                        setEditingZone(zone);
                        setShowForm(true);
                      }}
                      onDelete={handleDeleteZone}
                    />
                  </div>
                ))}
              </div>
            )}
          </>
        )}

        {/* Zone Form Modal */}
        {showForm && (
          <ZoneForm
            zone={editingZone}
            onSubmit={handleFormSubmit}
            onCancel={() => {
              setShowForm(false);
              setEditingZone(null);
            }}
          />
        )}
      </div>
    </div>
  );
};

export default Dashboard;